      subroutine aptconl (ind, np, pgcon, nerr)

ccbeg.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c                             SUBROUTINE APTCONL
c
c     call aptconl (ind, np, pgcon, nerr)
c
c     Version:  aptconl  Updated    2006 September 1 15:00.
c               aptconl  Originated 1989 March 2 15:30.
c
c     Author:   Arthur L. Edwards, LLNL, L-298, Telephone (925) 422-4123.
c
c     Refs:     1.  Peter J. Mohr and Barry M. Taylor, CODATA Recommended Values
c               of the Fundamental Physical Constants: 1998, Journal of
c               Physical and Chemical Reference Data, Vol 28, No. 6, 1999,
c               and Reviews of Modern Physics, Vol. 72, No. 2, 2000.
c
c               2.  "Metric Practice Guide E 380-72e", 1973, American
c               Society for Testing and Materials, 1916 Race Street,
c               Philadelphia, Pa 19103.
c
c               3.  Charles Seife, "A Slow Carousel Ride Gauges Gravity's
c               Pull', Science Vol 288, 2000 May 12.
c
c               4.  Peter J. Mohr and Barry N. Taylor, CODATA Recommended
c               Values of the Fundamental Physical Constants: 2002, to be
c               published:  <http://physics.nist.gov/constants>.
c               NOT YET IMPLEMENTED IN THIS SUBROUTINE.
c
c
c     Purpose:  To provide the best available values of the fundamental
c               physical constants and conversion factors of physics and
c               chemistry, in cm-g-shake-keV units.  See aptconm for SI units.
c               For each of np values of ind, the constant with local index ind
c               is returned in array pgcon.  The local indices are tabulated
c               below.  For definitions of the constants, see the full listing
c               of this subroutine, or reference 3 listed above.  If any value
c               of ind is not in the range from 1 to 180, the returned value of
c               pgcon will be -1.e+99.
c               Flag nerr indicates any input error, if not zero.
c
c             1 pga        37 pgfrtoam   73 pgk       109 pgmsubmu  145 pgratde
c             2 pgabi85    38 pgfrtog    74 pgketoam  110 pgmsubn   146 pgratdp
c             3 pgalpha    39 pgfrtojk   75 pgketofr  111 pgmsubp   147 pgratea
c             4 pgalphai   40 pgfrtoke   76 pgketojk  112 pgmub     148 pgrated
c             5 pgamtohz   41 pgfrtorc   77 pgketorc  113 pgmube    149 pgratemu
c             6 pgamtojk   42 pgg        78 pgkevtog  114 pgmubf    150 pgratep
c             7 pgamtoke   43 pggamma    79 pglamce   115 pgmubw    151 pgratmue
c             8 pgamtorc   44 pggamp     80 pglamceb  116 pgmud     152 pgratne
c             9 pgamu      45 pggampb    81 pglamcn   117 pgmudb    153 pgratnp
c            10 pgamud     46 pggampp    82 pglamcnb  118 pgmudn    154 pgratpe
c            11 pgamue     47 pggamppb   83 pglamcp   119 pgmue     155 pgratpmu
c            12 pgamumu    48 pggolden   84 pglamcpb  120 pgmueb    156 pgrctoam
c            13 pgamun     49 pggsube    85 pglosch   121 pgmuen    157 pgrctofr
c            14 pgamup     50 pggsubmu   86 pgmagfq   122 pgmumu    158 pgrctog
c            15 pgamutog   51 pggsubn    87 pgmdev    123 pgmumub   159 pgrctojk
c            16 pgastar    52 pggtoamu   88 pgmdj     124 pgmumun   160 pgrctoke
c            17 pgasube    53 pggtofr    89 pgmeev    125 pgmun     161 pgrmole
c            18 pgasubmu   54 pggtojk    90 pgmej     126 pgmune    162 pgrydb
c            19 pgatm      55 pggtokev   91 pgmmrde   127 pgmunf    163 pgrydbc
c            20 pgatomx    56 pggtorc    92 pgmmrdp   128 pgmunt    164 pgrydbe
c            21 pgbohr     57 pgh        93 pgmmremu  129 pgmuntb   165 pgrydbev
c            22 pgc        58 pghart     94 pgmmrep   130 pgmuntn   166 pgsate
c            23 pgcsub1    59 pghartev   95 pgmmrmup  131 pgmunw    167 pgsatez
c            24 pgcsub2    60 pghbar     96 pgmmrne   132 pgmup     168 pgsb
c            25 pgcunit    61 pghbarc    97 pgmmrnp   133 pgmupb    169 pgsia
c            26 pgcuxun    62 pghbarok   98 pgmmuev   134 pgmupn    170 pgsid220
c            27 pgdrohm    63 pghlen     99 pgmmuj    135 pgmupp    171 pgsigw
c            28 pge        64 pghmass   100 pgmnev    136 pgmuppb   172 pgsivolm
c            29 pgebase    65 pghok     101 pgmnj     137 pgmuppn   173 pgtemp
c            30 pgec       66 pghtime   102 pgmolh    138 pgmuvac   174 pgthomx
c            31 pgeoh      67 pgjfvr    103 pgmolhc   139 pgnsuba   175 pgubb
c            32 pgeome     68 pgjktoam  104 pgmoxun   140 pgombi85  176 pgv76bi
c            33 pgeomp     69 pgjktofr  105 pgmpev    141 pgqcirc   177 pgvmols
c            34 pgepsvac   70 pgjktog   106 pgmpj     142 pgqcirch  178 pgvmolz
c            35 pgerad     71 pgjktoke  107 pgmsubd   143 pgqhc     179 pgwien
c            36 pgfar      72 pgjktorc  108 pgmsube   144 pgqhr     180 pi
c
c     Input:    ind, np.
c
c     Output:   pgcon, nerr.
c
c     Glossary:
c
c     ind       Input    Indices of constants to be stored in array pgcon.
c                          If ind(n)<1 or ind(n)>180, -1.e99 will be returned
c                          for pgcon(n).  Size np.
c
c     nerr      Output   Error flag.  0 if no errors.
c                        1 if np is not positive.
c
c     np        Input    Size of arrays ind, pgcon.
c
c     pgcon     Output   Constants.  Value pgcon(n) is the value of the constant
c                          with index ind(n), if ind(n)>0 and ind(n)<181.
c                          Otherwise, pgcon(n) will be -1.e99.  Size np.
c
c     Definitions:
c
c               SI UNIT AND CM-G-SHAKE UNIT CONVERSION FACTORS
c
c                   Definitions and conversion of base units
c
c     Length or distance.  1 cm (centimeter) = 0.01 m (meter).
c
c     Mass.  1 g (gram) = 0.001 kg (kilogram).
c
c     Time.  1 sh (shake) = 1.0e-08 s (second).
c
c     Electric current.  1 labA = 1 (jerk / cm)**0.5 = 1.0e+09 A (ampere).
c                        1 A = 1 C / s = (1.0e-07 J / m)**0.5.
c
c     Temperature.  1 keV = 1.1604506e+07 K (Kelvin).
c                   1 K = 1 / 273.16 of the thermodynamic temperature of
c                   the triple point of water.
c     Note:  1988 NBS triple point of Gallium = 302.9169 K (1.7 ppm).
c
c                Definitions and conversion of derived units
c
c     Acceleration.  1 cm / sh**2 = 1.0e+14 m / s**2.
c
c     Area.  1 cm**2 = 1.0e-04 m**2.
c
c     Density.  1 g / cm**3 = 1.0e+03 kg / m**3.
c
c     Electric capacitance.  1 labF = 1 sh**2 / cm = 1.0e-07 F (farad).
c                            1 F = 1 A s / V = 1.0e-07 s**2 / m.
c
c     Electric inductance. 1 labH = 1 cm = 1.0e-09 H (henry).
c                          1 H = 1 V s / A = 1.0e+07 m.
c
c     Electric resistance.  1 labohm = 1 cm / sh = 0.1 ohm.
c                           1 ohm = 1 V / A = 1.0e+07 m / s.
c
c     Electric voltage.  1 labV = 1 (jerk cm / sh**2)**0.5 = 1.0e+08 V (volt).
c                        1 V = 1 W / A = (1.0e+07 J m / s**2)**0.5.
c
c     Electrical charge.  1 labC = 1 (jerk sh**2 / cm)**0.5 = 10 C (coulomb).
c                         1 C = 1 A s = (1.0e-07 J s**2 / m)**0.5.
c
c     Electrical conductance.  1 labS = 1 sh / cm = 10 S (siemen).
c                              1 S = 1 A / V = 1.0e-07 s / m.
c
c     Electron volts.  1 keV (kiloelectron volts) = 1000 eV (electron volts).
c                      1 eV = 1.602176462e-19 J.
c
c     Energy or work.  1 jerk = 1.0e+09 J (joule).
c                      1 J = 1 kg m**2 / s**2 = 1 N m (newton meter).
c
c     Force.  1 g cm / sh**2 = 1 jerk / cm = 1.0e+07 N (newton).
c             1 N = 1 Kg m / s**2 = 1 J / m.
c
c     Magnetic flux.  1 labWb = 1 (jerk cm)**0.5 = 1 Wb (weber).
c                     1 Wb = 1 V s = (1.0e+07 J m)**0.5.
c
c     Magnetic flux density.  1 labT = 1 (jerk / cm**3)**0.5 = 10,000 T (tesla).
c                             1 T = 1 Wb / m**2 = (1.0e+07 J / m**3)**0.5.
c
c     Power.  1 jerk / sh = 1.0e+17 W (watt)
c             1 W = 1 J / s = 1.0e-17 jerk / sh.
c
c     Pressure or energy density.  1 jerk / cm**3 = 1.0e+15 Pa (pascal).
c                                  1 Pa = 1 N / m**2 = 1 J / m**3.
c
c     Specific heat.  1 jerk / (g keV) = 8.617342e+04 J / (kg K).
c
c     Thermal conductivity.  1 jerk / (sh cm keV) = 8.617342e+11 W / (m K).
c
c     Velocity.  1 cm / sh = 1.0e+06 m / s.
c
c     Volume.  1 cm**3 = 1.0e-06 m**3.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccend.

c.... Dimensioned arguments.

      dimension ind     (1)           ! Index in array of local constants.
      dimension pgcon   (1)           ! Constant pgcon(n) = constant(ind(n)).

c.... Local variables.

      common /laptconl/
     &  pga(1),   pgabi85,  pgalpha,  pgalphai, pgamtohz, pgamtojk,
     &  pgamtoke, pgamtorc, pgamu,    pgamud,   pgamue,   pgamumu,
     &  pgamun,   pgamup,   pgamutog, pgastar,  pgasube,  pgasubmu,
     &  pgatm,    pgatomx,  pgbohr,   pgc,      pgcsub1,  pgcsub2,
     &  pgcunit,  pgcuxun,  pgdrohm,  pge,      pgebase,  pgec,
     &  pgeoh,    pgeome,   pgeomp,   pgepsvac, pgerad,   pgfar,
     &  pgfrtoam, pgfrtog,  pgfrtojk, pgfrtoke, pgfrtorc, pgg,
     &  pggamma,  pggamp,   pggampb,  pggampp,  pggamppb, pggolden,
     &  pggsube,  pggsubmu, pggsubn,  pggtoamu, pggtofr,  pggtojk,
     &  pggtokev, pggtorc,  pgh,      pghart,   pghartev, pghbar
      common /laptconl/
     &  pghbarc,  pghbarok, pghlen,   pghmass,  pghok,    pghtime,
     &  pgjfvr,   pgjktoam, pgjktofr, pgjktog,  pgjktoke, pgjktorc,
     &  pgk,      pgketoam, pgketofr, pgketojk, pgketorc, pgkevtog,
     &  pglamce,  pglamceb, pglamcn,  pglamcnb, pglamcp,  pglamcpb,
     &  pglosch,  pgmagfq,  pgmdev,   pgmdj,    pgmeev,   pgmej,
     &  pgmmrde,  pgmmrdp,  pgmmremu, pgmmrep,  pgmmrmup, pgmmrne,
     &  pgmmrnp,  pgmmuev,  pgmmuj,   pgmnev,   pgmnj,    pgmolh,
     &  pgmolhc,  pgmoxun,  pgmpev,   pgmpj,    pgmsubd,  pgmsube,
     &  pgmsubmu, pgmsubn,  pgmsubp,  pgmub,    pgmube,   pgmubf,
     &  pgmubw,   pgmud,    pgmudb,   pgmudn,   pgmue,    pgmueb
      common /laptconl/
     &  pgmuen,   pgmumu,   pgmumub,  pgmumun,  pgmun,    pgmune,
     &  pgmunf,   pgmunt,   pgmuntb,  pgmuntn,  pgmunw,   pgmup,
     &  pgmupb,   pgmupn,   pgmupp,   pgmuppb,  pgmuppn,  pgmuvac,
     &  pgnsuba,  pgombi85, pgqcirc,  pgqcirch, pgqhc,    pgqhr,
     &  pgratde,  pgratdp,  pgratea,  pgrated,  pgratemu, pgratep,
     &  pgratmue, pgratne,  pgratnp,  pgratpe,  pgratpmu, pgrctoam,
     &  pgrctofr, pgrctog,  pgrctojk, pgrctoke, pgrmole,  pgrydb,
     &  pgrydbc,  pgrydbe,  pgrydbev, pgsate,   pgsatez,  pgsb,
     &  pgsia,    pgsid220, pgsigw,   pgsivolm, pgtemp,   pgthomx,
     &  pgubb,    pgv76bi,  pgvmols,  pgvmolz,  pgwien,   pi

      common /laptconl/ indx          ! Index in external array.
      common /laptconl/ n             ! Index in external array.
cbugc***DEBUG begins.
cbug 9901 format (/ 'aptconl finding physical constants.  np=',i4 /
cbug     &  ('  n=',i4,' ind=',i3))
cbug      write ( 3, 9901) np, (n, ind(n), n = 1, np)
cbugc***DEBUG ends.

c.... The data.

c....                          GENERAL CONSTANTS

c....                         Numerical Constants

c.... Ratio of circumference to diameter of circle, pi.
      pi       = 3.14159265358979323  ! Dimensionless.    18 digits.

c.... Euler-Masceroni constant, gamma.
      pggamma  = 0.57721566490153386  ! Dimensionless.    17 digits.

c.... Golden Ratio, Phi = 0.5 * (1 + 5**0.5).
      pggolden = 1.61803398874989484  ! Dimensionless.    18 digits.

c.... Base of natural logarithms, e.
      pgebase  = 2.71828182845904523  ! Dimensionless.    18 digits.

c....                         Universal Constants

c.... Speed of light in vacuum, c (exact).
      pgc      = 299.792458           ! cm / sh.              exact.

c.... Unit system constant, c_unit (exact).
c....   Note:  (jerk/cm)/labA**2 = labH/cm.
      pgcunit  = 1.0                  ! (jerk/cm)/labA**2.    exact.

c.... Permeability of vacuum, mu_0 = 4 * pi * c_unit.
c....   Note:  (jerk/cm)/labA**2 = labH/cm.
      pgmuvac  = 12.5663706143591729  ! (jerk/cm)/labA**2.    0 ppt.

c.... Permittivity of vacuum, epsilon_0 = 1 / (mu_0 * c**2).
      pgepsvac = 8.8541878176203e-07  ! sh**2 / cm**2.     0.01 ppt.

c.... Newtonian constant of gravitation, G.
      pgg      = 6.6742300000000e-24  ! cm**3/(g sh**2).     14 ppm.

c.... Boltzmann constant in energy and electron volt units, k.
      pgk      = 1.6021764620000e-25  ! jerk / keV.          39 ppb.

c.... Planck constant, h.
      pgh      = 6.6260687600000e-35  ! jerk sh.             78 ppb.

c.... Planck constant in electron volts, h / k.
      pghok    = 4.1356672700000e-10  ! keV sh.              39 ppb.

c.... Planck constant / (2.0 * pi), hbar.
      pghbar   = 1.0545715960000e-35  ! jerk sh.             78 ppb.

c.... Planck constant / (2.0 * pi) in electron volts, hbar / k.
      pghbarok = 6.5821188900000e-11  ! keV sh.              39 ppb.

c.... Planck constant * velocity of light / (2.0 * pi), hbar * c
      pghbarc  = 3.1615261090182e-33  ! jerk cm.           0.60 ppm.

c.... Planck mass, m_P = (hbar * c / G)**0.5.
      pghmass  = 2.1764450000000e-05  ! g.                    7 ppm.

c.... Planck length, l_P = hbar / (m_P * c).
      pghlen   = 1.6162460000000e-33  ! cm.                   7 ppm.

c.... Planck time, t_P = l_P / c.
      pghtime  = 5.3912180000000e-36  ! sh.                   7 ppm.

c....                      Electromagnetic Constants

c.... Elementary charge, e.
      pge      = 1.6021764620000e-20  ! labC.                39 ppb.

c.... Elementary charge * velocity of light, e * c.
      pgec     = 4.8032041970000e-18  ! labC cm / sh.        38 ppb.

c.... Elementary charge / Planck constant, e / h.
      pgeoh    = 2.4179894910000e+14  ! labA / jerk.         78 ppb.

c.... Magnetic flux quantum, Phi_0 = h / (2 * e).
      pgmagfq  = 2.0678336360000e-15  ! labWb /labV sh.      39 ppb.

c.... Josephson frequency-voltage ratio, 2 * e / h.
      pgjfvr   = 4.8359789800000e+14  ! (cycles/sh)/labV.    39 ppb.

c.... Quantized Hall conductance, e**2 / h.
      pgqhc    = 3.8740458480000e-06  ! sh / cm.            3.7 ppb.

c.... Quantized Hall resistance, R_H = h / e**2.
      pgqhr    = 258128.07572         ! labohm.             3.7 ppb.

c.... Bohr magneton, mu_B = e * hbar / (2 * m_e).
      pgmub    = 9.2740089900000e-29  ! jerk / labT.         40 ppb.

c.... Bohr magneton in electron volts, mu_B / k.
      pgmube   = 5.7883817490000e-04  ! keV  / labT.        7.3 ppb.

c.... Bohr magneton in frequency units, mu_B / h.
      pgmubf   = 1.3996246240000e+06  ! (cycles/sh)/labT.    40 ppb.

c.... Bohr magneton in wavenumbers, mu_B / (h * c).
      pgmubw   = 4668.64521           ! (1/cm) / labT.       40 ppb.

c.... Nuclear magneton, mu_N = e * hbar / (2 * m_p).
      pgmun    = 5.0507831700000e-32  ! jerk / labT.         40 ppb.

c.... Nuclear magneton in electron volts, mu_N / k.
      pgmune   = 3.1524512380000e-07  ! keV  / labT.         40 ppb.

c.... Nuclear magneton in frequency units, mu_N / h.
      pgmunf   = 762.259396           ! (cycles/sh)/labT.    40 ppb.

c.... Nuclear magneton in wavenumbers, mu_N / (h * c).
      pgmunw   = 2.54262366           ! (1/cm) / labT.       40 ppb.


c....                          ATOMIC CONSTANTS

c....                               General

c.... Fine-structure constant, alpha = mu_0 * c * e**2 / (2 * h).
      pgalpha  = 0.00729735253595     ! Dimensionless.      0.7 ppb.

c.... Inverse fine-structure constant, 1 / alpha.
      pgalphai = 137.035999710        ! Dimensionless.      0.7 ppb.

c.... Rydberg constant in wavenumbers, R_infinity = Ry.
      pgrydb   = 109737.31568549      ! 1 / cm.             7.6 ppt.

c.... Rydberg constant in frequency units, R_infinity * c.
      pgrydbc  = 3.2898419603680e+07  ! 1 / sh.             7.6 ppt.

c.... Rydberg constant in energy units, R_infinity * h * c.
      pgrydbe  = 2.1798719000000e-27  ! jerk.                78 ppb.

c.... Rydberg constant in electron volts, R_infinity * h * c / pgk.
      pgrydbev = 0.01360569172        ! keV.                 39 ppb.

c.... Bohr radius, a_0 = alpha / (4 * pi * r_infinity).
      pgbohr   = 5.2917720830000e-09  ! cm.                 3.7 ppb.

c.... Atomic cross section, pi * a_0**2.
      pgatomx  = 8.7973553430000e-17  ! cm**2.              7.4 ppb.

c.... Hartree energy, E_h = 2 * R_infinity * h * c.
c....                         = e**2 / (4 * pi * epsilon_0 * a_0).
      pghart   = 4.3597438100000e-27  ! jerk.                78 ppb.

c.... Hartree energy in electron volts, E_h / k.
      pghartev = 0.0272113834         ! keV.                 39 ppb.

c.... Quantum of circulation, h / m_e or h / (2 * m_e).
      pgqcirc  = 7.2738950320000e-08  ! cm**2 / sh.         7.3 ppb.
      pgqcirch = 3.6369475160000e-08  ! cm**2 / sh.         7.3 ppb.

c....                              Electron

c.... Electron mass, m_e.
      pgmsube  = 9.1093818800000e-28  ! g.                   79 ppb.

c.... Electron molar mass, M_e.
      pgamue   = 0.0005485799110      ! g / mol /amu.       2.1 ppb.

c.... Electron mass in energy units.
      pgmej    = 8.1871041400000e-23  ! jerk.                79 ppb.

c.... Electron mass in electron volts.
      pgmeev   = 510.998902           ! keV.                 40 ppb.

c.... Electron-muon mass ratio, m_e / m_mu.
      pgratemu = 0.00483633210        ! Dimensionless.       30 ppb.

c.... Electron-proton mass ratio, m_e / m_p.
      pgratep  = 0.0005446170232      ! Dimensionless.      2.1 ppb.

c.... Electron-deuteron mass ratio, m_e / m_d.
      pgrated  = 0.00027244371170     ! Dimensionless.      2.1 ppb.

c.... Electron-alpha-particle mass ratio, m_e / m_alpha.
      pgratea  = 0.00013709335611     ! Dimensionless.      2.1 ppb.

c.... Electron specific (negative) charge, e / m_e.
      pgeome   = 1.7588201740000e+07  ! labC / g.            40 ppb.

c.... Electron Compton wavelength, lambda_C = h / (m_e * c).
      pglamce  = 2.4263102150000e-10  ! cm.                 7.3 ppb.

c.... Electron Compton wavelength / (2.0 * pi), lambdabar_C.
      pglamceb = 3.8615926420000e-11  ! cm.                 7.3 ppb.

c.... Electron classical radius, r_e.
      pgerad   = 2.8179402850000e-13  ! cm.                  11 ppb.

c.... Thomson cross section, sigma_e.
      pgthomx  = 0.6652458540000e-24  ! cm**2.               22 ppb.

c.... Electron magnetic moment, mu_e.
      pgmue    = 9.2847636200000e-29  ! jerk / labT.         40 ppb.

c.... Electron magnetic moment in Bohr magnetons, mu_e / mu_B.
      pgmueb   = 1.00115965218085     ! Dimensionless.      0.8 ppt.

c.... Electron magnetic moment in nuclear magnetons, mu_e / mu_N.
      pgmuen   = 1838.2819660         ! Dimensionless.      2.1 ppb.

c.... Electron magnetic moment anomaly, a_e.
      pgasube  = 0.00115965218085     ! Dimensionless.      0.8 ppb.

c.... Electron g-factor, g_e = 2 * (1 + a_e).
      pggsube  = 2.00231930436170     ! Dimensionless.      0.8 ppt.

c.... Electron-muon magnetic moment ratio, mu_e / mu_mu.
      pgmmremu = 206.7669720          ! Dimensionless.       30 ppb.

c.... Electron-proton magnetic moment ratio, mu_e / mu_p.
      pgmmrep  = 658.2106875          ! Dimensionless.       10 ppb.

c....                                Muon

c.... Muon mass, m_mu.
      pgmsubmu = 1.8835310900000e-25  ! g.                   84 ppb.

c.... Muon molar mass, M_mu.
      pgamumu  = 0.1134289168         ! g / mol /amu.        30 ppb.

c.... Muon mass in energy units.
      pgmmuj   = 1.6928333200000e-20  ! jerk.                84 ppb.
      pgmmuev  = 105658.3568          ! keV.                 49 ppb.

c.... Muon-electron mass ratio, m_mu / m_e.
      pgratmue = 206.7682657          ! Dimensionless.       30 ppb.

c.... Muon magnetic moment, mu_mu.
      pgmumu   = 4.4904481300000e-31  ! jerk / labT.         49 ppb.

c.... Muon magnetic moment in Bohr magnetons, mu_mu / mu_B.
      pgmumub  = 0.00484197085        ! Dimensionless.       30 ppb.

c.... Muon magnetic moment in nuclear magnetons, mu_mu / mu_N.
      pgmumun  = 8.89059770           ! Dimensionless.       30 ppb.

c.... Muon magnetic moment anomaly, a_mu.
      pgasubmu = 0.00116591602        ! Dimensionless.     0.55 ppm.

c.... Muon g-factor, g_mu = 2 * (1 + a_mu).
      pggsubmu = 2.0023318320         ! Dimensionless.     0.64 ppb.

c.... Muon-proton magnetic moment ratio, mu_mu / mu_p.
      pgmmrmup = 3.18334539           ! Dimensionless.       32 ppb.

c....                               Proton

c.... Proton mass, m_p.
      pgmsubp  = 1.6726215800000e-24  ! g.                   79 ppb.

c.... Proton molar mass, M_p.
      pgamup   = 1.00727646688        ! g / mol /amu.      0.13 ppb.

c.... Proton mass in energy units.
      pgmpj    = 1.5032773100000e-19  ! jerk.                79 ppm.

c.... Proton mass in electron volts.
      pgmpev   = 938271.998           ! keV.                 40 ppb.

c.... Proton-electron mass ratio, m_p / m_e.
      pgratpe  = 1836.1526675         ! Dimensionless.      2.1 ppb.

c.... Proton-muon mass ratio, m_p / m_mu.
      pgratpmu = 8.88024408           ! Dimensionless.       30 ppb.

c.... Proton specific charge, e / m_p.
      pgeomp   = 9578.83408           ! labC / g.            40 ppb.

c.... Proton Compton wavelength, lambda_C,p = h / (m_p * c).
      pglamcp  = 1.3214098470000e-13  ! cm.                 7.6 ppb.

c.... Proton Compton wavelength / (2.0 * pi), lambdabar_C,p.
      pglamcpb = 2.1030890890000e-14  ! cm.                 7.6 ppb.

c.... Proton magnetic moment, mu_p.
      pgmup    = 1.4106066330000e-31  ! jerk / labT.         41 ppb.

c.... Proton magnetic moment in Bohr magnetons, mu_p / mu_B.
      pgmupb   = 0.001521032203       ! Dimensionless.       10 ppb.

c.... Proton magnetic moment in nuclear magnetons, mu_p / mu_N.
      pgmupn   = 2.792847337          ! Dimensionless.       10 ppb.

c.... Proton magnetic moment diamagnetic shielding correction, sigma_water,
c....   in pure water, spherical sample, 298.15 K, 1 - mu'_p / mu_p.
      pgsigw   = 2.5687000000000e-05  ! Dimensionless.      570 ppm.

c.... Shielded proton moment, mu'_p (in water, spherical, 298.15 K).
      pgmupp   = 1.4105703990000e-31  ! jerk / labT.         42 ppb.

c.... Shielded proton moment in Bohr magnetons, mu'_p / mu_B.
      pgmuppb  = 0.001520993132       ! Dimensionless.       11 ppb.

c.... Shielded proton moment in nuclear magnetons, mu'_p / mu_N.
      pgmuppn  = 2.792775597          ! Dimensionless.       11 ppb.

c.... Proton gyromagnetic ratio, gamma_p = 2 * mu_p / hbar.
      pggamp   = 26752.2212           ! (1/sh) / labT.       41 ppb.

c.... Proton gyromagnetic ratio / (2.0 * pi), gamma_p / (2 * pi).
      pggampb  = 42577.4825           ! cycles / (sh labT).  41 ppb.

c.... Shielded proton gyromagnetic ratio, gamma'_p = 2 * mu'_p / hbar.
      pggampp  = 26751.5341           ! (1/sh) / labT.       42 ppb.

c.... Shielded proton gyromagnetic ratio / (2.0 * pi), gamma'_p / (2 * pi).
      pggamppb = 4257.63888           ! cycles / (sh labT).  42 ppb.

c....                               Neutron

c.... Neutron mass, m_n.
      pgmsubn  = 1.6749271600000e-24  ! g.                   79 ppb.

c.... Neutron molar mass, M_n.
      pgamun   = 1.00866491578        ! g / mol /amu.      0.54 ppb.

c.... Neutron mass in energy units.
      pgmnj    = 1.5053494600000e-19  ! jerk.                79 ppb.
      pgmnev   = 939565.330           ! keV.                 40 ppb.

c.... Neutron-electron mass ratio, m_n / m_e.
      pgratne  = 1838.6836550         ! Dimensionless.      2.2 ppb.

c.... Neutron-proton mass ratio, m_n / m_p.
      pgratnp  = 1.00137841887        ! Dimensionless.     0.58 ppb.

c.... Neutron Compton wavelength, lambda_C,n = h / (m_n * c).
      pglamcn  = 1.3195908980000e-13  ! cm.                  76 ppb.

c.... Neutron Compton wavelength / (2.0 * pi), lambdabar_C,n.
      pglamcnb = 2.1001941420000e-14  ! cm.                 7.6 ppb.

c.... Neutron magnetic moment, mu_n (scaler, opposite from proton).
      pgmunt   = 9.6623640000000e-32  ! jerk / labT.       0.24 ppm.

c.... Neutron magnetic moment in Bohr magnetons, mu_n / mu_B.
      pgmuntb  = 0.00104187563        ! Dimensionless.     0.24 ppm.

c.... Neutron magnetic moment in nuclear magnetons, mu_n / mu_N.
      pgmuntn  = 1.91304272           ! Dimensionless.     0.24 ppm.

c.... Neutron-electron magnetic moment ratio, mu_n / mu_e.
      pgmmrne  = 0.00104066882        ! Dimensionless.     0.24 ppm.

c.... Neutron-proton magnetic moment ratio, mu_n / mu_p.
      pgmmrnp  = 0.68497934           ! Dimensionless.     0.24 ppm.

c....                              Deuteron

c.... Deuteron mass, m_d.
      pgmsubd  = 3.3435830900000e-24  ! g.                   79 ppb.

c.... Deuteron molar mass, M_d.
      pgamud   = 2.01355321271        ! g / mol /amu.      0.17 ppb.

c.... Deuteron mass in energy units.
      pgmdj    = 3.0050626200000e-19  ! jerk.                79 ppb.

c.... Deuteron mass in electron volt units.
      pgmdev   = 1875612.762          ! keV.                 40 ppb.

c.... Deuteron-electron mass ratio, m_d / m_e.
      pgratde  = 3670.4829550         ! Dimensionless.       40 ppb.

c.... Deuteron-proton mass ratio, m_d / m_p.
      pgratdp  = 1.99900750083        ! Dimensionless.      0.2 ppb.

c.... Deuteron magnetic moment, mu_d (scaler).
      pgmud    = 4.3307345700000e-32  ! jerk / labT.         42 ppb.

c.... Deuteron magnetic moment in Bohr magnetons, mu_d / mu_B.
      pgmudb   = 0.0004669754556      ! Dimensionless.       11 ppb.

c.... Deuteron magnetic moment in nuclear magnetons, mu_d / mu_N.
      pgmudn   = 0.8574382284         ! Dimensionless.       11 ppb.

c.... Deuteron-electron magnetic moment ratio, mu_d / mu_e.
      pgmmrde  = 0.0004664345537      ! Dimensionless.       11 ppb.

c.... Deuteron-proton magnetic moment ratio, mu_d / mu_p.
      pgmmrdp  = 0.3070122083         ! Dimensionless.       15 ppb.


c....                     PHYSICO-CHEMICAL CONSTANTS

c.... Avogadro constant, N_A = L = 12 / mass of C-12.
      pgnsuba  = 6.0221419900000e+23  ! # / mol = amu / g    79 ppb.

c.... Atomic mass constant, m_u = 1 / N_A = 1/12 mass of C-12.
      pgamu    = 1.6605387300000e-24  ! g / amu.             79 ppb.

c.... Faraday constant, F.
      pgfar    = 9648.53415           ! labC / mol.          40 ppb.

c.... Molar Planck constant, N_A * h.
      pgmolh   = 3.9903126890000e-11  ! jerk sh / mol.      7.6 ppb.

c.... Molar Planck constant * c, N_A * h * c.
      pgmolhc  = 1.1962656492000e-08  ! jerk cm / mol.      7.6 ppb.

c.... Molar gas constant, R.
      pgrmole  = 0.0964853415         ! jerk / (mol keV).    40 ppb.

c.... Molar volume (ideal gas), V_m = R*T/p.
c....   T = pgtempt = 273.15K,  pgtemp = 0.02353839 eV = 2.353839e-5 keV,
c....   p = pgatm = 101325 Pa = 1.10325e-10 jerk / cm**3.
      pgvmols  = 22413.996            ! cm**3 / mol.        1.7 ppm.

c.... Molar volume (ideal gas), V_m = R*T/p.
c....   T = pgtempt = 273.15K,  pgtemp = 0.02353839 eV = 2.353839e-5 keV,
c....   p = 0.986923 atm = 100000 Pa = 1.0e-10 jerk / cm**3.
      pgvmolz  = 22710.981            ! cm**3 / mol.        1.7 ppm.

c.... Loschmidt constant, n_0 = N_A / V_m.
c....   T = 273.15 K = 0.0 Celsius = 0.02353839 eV = 2.353839e-5 keV,
c....   p = 101325.0 Pa = 1.0 atm = 1.01325e-10 jerk / cm**3.
      pglosch  = 2.6867775000000e+19  ! # / cm**3.          1.7 ppm.

c.... Sackur-Tetrode constant, S_0 / R.
c....   The entropy of an ideal monatomic gas of relative atomic weight A is
c....   S = S_0 + 1.5 * R * alog (A) - R * alog (p / p_0) +
c....       2.5 * R * alog (T / K).
c....   T_1 = 1 K, p_0 = 101325 1.0e-15 jerk / cm**3.
      pgsate   = -1.1648678           ! Dimensionless.      3.7 ppm.

c....   T_1 = 1 K, p_0 = 100000 1.0e-15 jerk / cm**3.
      pgsatez  = -1.1517048           ! Dimensionless.      3.8 ppm.

c.... Black body constant, u_bb.
      pgubb    = 0.0021127638         ! jerk/(cm**3 keV**4).  3 ppm.

c.... Radiation energy constant, u_a.
      pga(1)   = 0.013720160          ! jerk/(cm**3 keV**4).  3 ppm.

c.... Stefan-Boltzmann constant, sigma.
      pgsb     = 1.0283012            ! jerk/(sh cm**2 keV**4)3 ppm.

c.... First radiation constant, c_1 = 2.0 * pi * h * c**2.
      pgcsub1  = 3.7417710700000e-29  ! jerk cm**2 / sh.     78 ppb.

c.... Second radiation constant in electron volt units, c_2 = h * c / k.
      pgcsub2  = 1.2398418570000e-07  ! keV cm.              39 ppb.

c.... Wien displacement law constant in electron volt units, b.
      pgwien   = 2.4971076000000e-08  ! keV cm.            0.30 ppm.


c....                           STANDARD VALUES

c.... Standard acceleration of gravity, g_n.
      pggsubn  = 9.8066500000000e-14  ! cm / sh**2.           exact.

c.... Standard atmosphere, p_0 = 1 atmosphere.
      pgatm    = 1.0132500000000e-10  ! jerk / cm**3.         exact.

c.... Standard temperature, T_0 = 0.0 Celsius.
      pgtemp   = 2.3538270000000e-05  ! keV.                1.7 ppm.

c....                 "As-maintained" Electrical Standards

c.... BIPM maintained ohm, omega_69-BI (1 Jan 1985) = omega_BI85.
      pgombi85 = 0.9999984370000e+01  ! labohm.              50 ppb.  (old)

c.... Drift rate of omega_69-BI, d[omega_69-BI] / dt.
      pgdrohm  = -5.660000000000e-07  ! labohm / yr?.              ?  (old)

c.... BIPM maintained volt, V_76-BI = 4.83594e+14 * h / (2 * e).
      pgv76bi  = 0.9999924100000e-08  ! labV.              0.30 ppm.  (old)

c.... BIPM maintained ampere, A_BI85 = V_76-BI / omega_69-BI.
      pgabi85  = 0.9999939700000e-09  ! labA.              0.30 ppm.  (old)

c....                           X-Ray Standards

c.... Cu x-unit, xu_CuKalpha_1:
c....   lambda_CuKalpha_1 = 1537.400 xu.
      pgcuxun  = 1.0020770300000e-11  ! cm.                0.28 ppm.

c.... Mo x-unit, xu_MoKalpha_1:
c....   lambda_MoKalpha_1 = 707.831 xu.
      pgmoxun  = 1.0020995900000e-11  ! cm.                0.53 ppm.

c.... A^*:  lambda_WKalpha_1 = 0.209100 A^*.
      pgastar  = 1.0000150100000e-08  ! cm.                0.90 ppm.

c.... Lattice spacing of Si, a (in vacuum, 295.65 K).
      pgsia    = 5.4310208800000e-08  ! cm.                  29 ppb.

c.... Lattice spacing of Si, d_220 (in vacuum, 295.65 K) = a / 8**0.5.
      pgsid220 = 1.9201558450000e-08  ! cm.                  29 ppb.

c.... Molar volume of Si, V_m (Si) = M(Si) / rho(Si) = N_A a**3 / 8.
      pgsivolm = 12.0588369           ! cm**3 / mol.       0.12 ppm.


c....                      ENERGY CONVERSION FACTORS

c....                     Conversion to g mass units

c.... Equivalent:  1 / L.
      pgamutog = 1.6605387300000e-24  ! g  / amu.            79 ppb.

c.... Equivalent:  1 / c**2.
      pgjktog  = 1.1126500560536e-05  ! g  / jerk.            0 ppt.

c.... Equivalent:  k / c**2.
      pgkevtog = 1.7826617310000e-30  ! g  / keV.            39 ppb.

c.... Equivalent:  h / c.
      pgrctog  = 2.2102186300000e-37  ! g  / (1/cm).         77 ppb.

c.... Equivalent:  h / c**2.
      pgfrtog  = 7.3724957800000e-40  ! g  / (1/sh).         79 ppb.

c....                   Conversion to atomic mass units

c.... Equivalent:  L.
      pggtoamu = 6.0221419900000e+23  ! amu / g.             79 ppb.

c.... Equivalent:  L / c**2.
      pgjktoam = 6.7005366200000e+18  ! amu / jerk.          79 ppb.

c.... Equivalent:  L*k / c**2.
      pgketoam = 1.0735442060000e-06  ! amu / keV.           40 ppb.

c.... Equivalent:  h*L / c.
      pgrctoam = 1.3310250420000e-13  ! amu / (1/cm).        78 ppb.

c.... Equivalent:  h*L / c**2.
      pgfrtoam = 4.4398216370000e-16  ! amu / (1/sh).        78 ppb.

c....                  Conversion to jerk energy units

c.... Equivalent:  c**2.
      pggtojk  = 89875.517873681764   ! jerk / g.             0 ppt.

c.... Equivalent:  c**2 / L.
      pgamtojk = 1.4924177800000e-19  ! jerk / amu.          79 ppb.

c.... Equivalent:  k.
      pgketojk = 1.6021764620000e-25  ! jerk / keV.          39 ppb.

c.... Equivalent:  h * c.
      pgrctojk = 1.9864454400000e-32  ! jerk / (1/cm).       81 ppb.

c.... Equivalent:  h.
      pgfrtojk = 6.6260687600000e-35  ! jerk / (1/sh).       78 ppb.

c....                  Conversion to electron volt units

c.... Equivalent:  c**2 / k.
      pggtokev = 5.6095892100000e+29  ! keV / g.             39 ppb.

c.... Equivalent:  c**2/(L*k).
      pgamtoke = 931494.013           ! keV / amu.           40 ppb.

c.... Equivalent:  1 / k.
      pgjktoke = 6.2415097400000e+24  ! keV / jerk.          38 ppb.

c.... Equivalent:  h * c / k.
      pgrctoke = 1.2398418570000e-07  ! keV / (1/cm).        39 ppb.

c.... Equivalent:  h / k.
      pgfrtoke = 4.1356672700000e-10  ! keV / (1/sh).        39 ppb.

c....                   Conversion to wavenumber units

c.... Equivalent:  c / h.
      pggtorc  = 4.5244392900000e+36  ! (1/cm) / g.          77 ppb.

c.... Equivalent:  c / (L * h).
      pgamtorc = 7.5130066580000e+12  ! (1/cm) / amu.       7.6 ppb.

c.... Equivalent:  1 / (h * c).
      pgjktorc = 5.0341176200000e+31  ! (1/cm) / jerk.       77 ppb.

c.... Equivalent:  k / (h * c).
      pgketorc = 8065544.77           ! (1/cm) / keV.        39 ppb.

c.... Equivalent:  1 / c.
      pgfrtorc = 0.00333564095198152  ! (1/cm) / (1/sh).      4 ppt.

c....                    Conversion to frequency units

c.... Equivalent:  c**2 / h.
      pggtofr  = 1.3563927700000e+39  ! (1/sh) / g.          81 ppb.

c.... Equivalent:  c**2 / (L*h).
      pgamtohz = 2.2523427330000e+15  ! (1/sh) / amu.       7.5 ppb.

c.... Equivalent:  1 / h.
      pgjktofr = 1.5091905000000e+34  ! (1/sh) / jerk.       79 ppb.

c.... Equivalent:  k / h.
      pgketofr = 2.4179894910000e+09  ! (1/sh) / keV.        39 ppb.

c.... Equivalent:  c.
      pgrctofr = 299.792458           ! (1/sh) / (1/cm).      exact.

c.... Test for input errors.

      nerr = 0
      if (np .le. 0) then
        nerr = 1
        go to 210
      endif

c.... Find the constants.

      do 110 n = 1, np                ! Loop over returned constants.

        if ((ind(n) .ge. 1) .and.
     &      (ind(n) .le. 180)) then
          indx = ind(n)
          pgcon(n) = pga(indx)
        else
          pgcon(n) = -1.e99
        endif

  110 continue                        ! End of loop over returned constants.
cbugc***DEBUG begins.
cbug 9902 format (/ 'aptconl results:' /
cbug     &  ('  n=',i4,' ind=',i3,' pgcon=',1pe22.14))
cbug      write ( 3, 9902) (n, ind(n), pgcon(n), n = 1, np)
cbugc***DEBUG ends.

  210 return

c.... End of subroutine aptconl.      (+1 line.)
      end
